@@ -34,6 +34,8 @@ module Data.Conduit.Internal.Conduit
     , yieldM
     , leftover
     , runConduit
+    , runConduitPure
+    , runConduitRes
     , fuse
     , connect
       -- ** Composition
@@ -97,6 +99,7 @@ import Control.Monad.State.Class(MonadState(..))
 import Control.Monad.Trans.Class (MonadTrans (lift))
 import Control.Monad.IO.Unlift (MonadIO (liftIO), MonadUnliftIO, withRunInIO)
 import Control.Monad.Primitive (PrimMonad, PrimState, primitive)
+import Data.Functor.Identity (Identity, runIdentity)
 import Data.Void (Void, absurd)
 import Data.Monoid (Monoid (mappend, mempty))
 import Data.Semigroup (Semigroup ((<>)))
@@ -730,6 +733,22 @@ fuse = (=$=)
 -- Equivalent to 'fuse' and '=$=', however the latter is deprecated and will
 -- be removed in a future version.
 --
+-- Note that, while this operator looks like categorical composition
+-- (from "Control.Category"), there are a few reasons it's different:
+--
+-- * The position of the type parameters to 'ConduitT' do not
+--   match. We would need to change @ConduitT i o m r@ to @ConduitT r
+--   m i o@, which would preclude a 'Monad' or 'MonadTrans' instance.
+--
+-- * The result value from upstream and downstream are allowed to
+--   differ between upstream and downstream. In other words, we would
+--   need the type signature here to look like @ConduitT a b m r ->
+--   ConduitT b c m r -> ConduitT a c m r@.
+--
+-- * Due to leftovers, we do not have a left identity in Conduit. This
+--   can be achieved with the underlying @Pipe@ datatype, but this is
+--   not generally recommended. See <https://stackoverflow.com/a/15263700>.
+--
 -- @since 1.2.8
 (.|) :: Monad m
      => ConduitM a b m () -- ^ upstream
@@ -1231,3 +1250,23 @@ fuseUpstream up down = fmap fst (fuseBoth up down)
 {-# RULES "conduit: leftover l >> p" forall l (p :: ConduitT i o m r). leftover l >> p =
     ConduitT (Leftover (unConduitT p) l) #-}
     -}
+
+-- | Run a pure pipeline until processing completes, i.e. a pipeline
+-- with @Identity@ as the base monad. This is equivalient to
+-- @runIdentity . runConduit@.
+--
+-- @since 1.2.8
+runConduitPure :: ConduitT () Void Identity r -> r
+runConduitPure = runIdentity . runConduit
+{-# INLINE runConduitPure #-}
+
+-- | Run a pipeline which acquires resources with @ResourceT@, and
+-- then run the @ResourceT@ transformer. This is equivalent to
+-- @runResourceT . runConduit@.
+--
+-- @since 1.2.8
+runConduitRes :: MonadUnliftIO m
+              => ConduitT () Void (ResourceT m) r
+              -> m r
+runConduitRes = runResourceT . runConduit
+{-# INLINE runConduitRes #-}
